# Bibliotheque correspondant au document
# Raisonnement dans l'analyse de donnees
# (cl) L.-O. Pochon, IPE-UniNe

VersionIPErad <- "v1.4.2 octobre 2012"   
cat(VersionIPErad, " (cc) L.-O. Pochon & IPE-UniNE\n")

# Representation triangulaire

init.RT <- function(lab=NULL) {
  init.graph(lab)
}

init.graph <- function(lab=NULL) {
  r3 <<- sqrt(3)
  hmax <<- r3/2
  plot.new()
  lines(c(0,0.5,1,0),c(0,r3/2,0,0))
  if(!is.null(lab)) {
    text(c(0,1,0.5),c(-0.02,-0.02,r3/2+0.02),lab)
  }
}

pos.x <- function(ksi,eta) {
# calcule la coordonne y d'un point ksi, eta 
# pos.y = ksi
    (2* eta + ksi) / 2
}

marque <- function(ksi,eta,c="x",col="red"){
        text(pos.x(ksi,eta),hmax*ksi,c,col=col)
}

# Representation a partir d'un data.frame
# selon la structure id val1 val2 1-val1-val2

affiche.RT.old <- function(df,ordre=1,pct=T){
# df: dataframe, ordre, pct: effectif ou frequence
# pour charger le data.frame: df <- read.table(fichier)
# format du fichier
# "var1" "var2" "var3"
# "id" v1 v2 v3

   noms <- dimnames(df)[[1]]
   i <- 1
   for(nom in noms) {
      ksi <- df[i,1]
      eta <- df[i,2]
      if(!pct) {
         n <- ksi + eta +df[i,3]
         ksi <- ksi/n
         eta <- eta/n
      }
      marque(ksi,eta,c=nom)
      i <- i+1
   }
}
affiche.RT <- function(df,ordre=1,pct=T){
# df: dataframe, ordre, pct: effectif ou frequence
# pour charger le data.frame: df <- read.table(fichier)
# format du fichier
# "var1" "var2" "var3"
# "id" v1 v2 v3

   noms <- dimnames(df)[[1]]
   ksi <- df[,1]
   eta <- df[,2]
   if(!pct) {
      n <- ksi + eta +df[,3]
      ksi <- ksi/n
      eta <- eta/n
   }
   marque(ksi,eta,noms)
}

# graphique radar

radar <- function(tab,labels=dimnames(tab)[[2]],glabels=dimnames(tab)[[1]],main=NULL,couleur = c("red","blue","green","violet","orange","maroon"),
         relative=FALSE,rayon=TRUE) {
  profil(tab,labels, glabels,main,couleur,relative,rayon,radar=TRUE)
}

profil <- function(tab,labels=dimnames(tab)[[2]],glabels=dimnames(tab)[[1]],main=NULL,couleur = c("red","blue","green","violet","orange","maroon"),
         relative=FALSE,rayon=TRUE,radar=TRUE) {
# tab matrice, chaque ligne va correspondre a une etoile
# relative = TRUE le cercle maximum correspond au % max, sinon 1
  margen <- c(1,1,1,1)

  old.par <- par(no.readonly = TRUE)
  par(mar=margen)
  plot.new()  
  on.exit(par(old.par))

  nc <- ncol(tab)
  nl <- nrow(tab)

  if(nc<2) {
    cat("Nombre de colonnes insuffisant")
    return(-1)
  }

  suml <- apply(tab,1,sum)
  tfre <- diag(1/suml) %*% as.matrix(tab)

  maxf = max(tfre)

  if(maxf>0.75 | !relative ) {
    ncercle <- 4
    rcercle <- c(0.25,0.5,0.75,1)
  } else if (maxf>0.5) {
    ncercle <- 3
    rcercle <- c(0.25, 0.5, 0.75) 
  } else if (maxf>0.4) {
    ncercle <- 5
    rcercle <- c(0.1, 0.2, 0.3,0.4,0.5) 
  } else if (maxf>0.3) {
    ncercle <- 4
    rcercle <- c(0.1, 0.2, 0.3,0.4) 
  } else {
    ncercle <- 3
    rcercle <- c(0.1, 0.2, 0.3) 
  }

  fen <- rcercle[ncercle] 
  
  ylim <- c(-fen,fen)
  xlim <- c(-fen,fen)

  if(nc==2 | !radar) {
    xlim <- c(-1,1)
#
   ylim <- c(-fen,0.2)

#    oy <- fen / 2
   oy <- fen - 0.1

    plot(NA, NA, type="n", xlim=xlim, ylim=ylim, xlab="", ylab="", main=main, axes=FALSE)

# lignes horizontales
    pfg <- par("fg")
    par(fg="grey")
    rcercle <- c(0,rcercle)

    for(i in 0:ncercle) {
      lines(c(-0.75,0.75),c(rcercle[i+1]-oy,rcercle[i+1]-oy))
    }
    par(fg=pfg)

# labels
    labval <- paste(format(100*rcercle,digits=2),"%")
    text(rep(-0.9,ncercle+1),rcercle-oy,labval)
    text(rep(0.9,nl),0.1 - 0.1*(1:nl),labels=glabels,col=couleur[1:nl])
#    text(rep(0.9,nl),fen- 0.1*(1:nl),labels=glabels,col=couleur[1:nl])

    seg <- 1.4 /(nc-1)
    text((0:(nc-1))*seg - 0.7,rep(-oy,nc),labels,cex=1.5)

    pfg <- par("fg")

    for(i in 1:nl) {
      par(fg=couleur[i])
      lines((0:(nc-1))*seg - 0.7,tfre[i,]-oy)
    }
    par(fg=pfg)
    
    if(rayon) {
      lty <- par("lty")
      par(lty=3)
      for(i in 1:nc) {
        lines(c((i-1)*seg - 0.7,(i-1)*seg - 0.7),c(-oy,fen-oy)) 
#        lines(c((i-1)*seg - 0.7,(i-1)*seg - 0.7),c(-oy,-oy)) 
      }
      par(lty=lty) 
    }

  } else {
    
    plot(NA, NA, type="n", xlim=xlim, ylim=ylim, xlab="", ylab="", main=main, axes=FALSE)

    pfg <- par("fg")
    par(fg="grey")
    symbols(rep(0,ncercle),rep(0,ncercle),circles=rcercle,xlab="", ylab="",inches=F,add=T)
    par(fg=pfg)

    labval <- paste(format(100*rcercle,digits=2),"%")
    text(rcercle,rep(-fen,ncercle),labval)

    text(rep(-fen,nl),fen- 0.1*(1:nl),labels=glabels,col=couleur[1:nl])

# labels
    delta <- (0:(nc-1))*2*pi/nc
    text(fen*cos(delta),fen*sin(delta),labels,cex=1.5)
 
    if(rayon) {
      lty <- par("lty")
      par(lty=3)
      for(d in delta) {
        lines(c(0,fen*cos(d)),c(0,fen*sin(d))) 
      }
      par(lty=lty) 
    }

    symbols(rep(0,nl),rep(0,nl),stars=tfre,xlab="",ylab="",inches=F,add=T,fg=couleur[1:nl])
  }
  invisible()
}

# Statistique non parametrique

S.kendall <- function(tab) {
   
# nb colonnes
# nc <- dim(tab)[2]
# nb lignes
   nl <- nrow(tab)
# somme des lignes: margin.table(tab, 1) 
   if(nl==2) {
     S <- prod.S(tab[1,],tab[2,])
   } else {
     S <- 0
     for(i in 1:(nl-2)) {
        S <- S + prod.S(tab[i,],apply(tab[(i+1):nl,],2,sum))
     }
     S <- S + prod.S(tab[nl-1,],tab[nl,])
   }
   S
} 

prod.S <- function(v,w) {
   n <- length(v) 
   m2 <- matrix(rep(1,n*n),n,n,byrow=T)
   m2[lower.tri(m2,diag=T)] <- 0
   F <- m2 - t(m2)
   t(v) %*% F %*% w
}

P.kendall <- function(tab) {
   
# nb colonnes
  nc <- dim(tab)[2]
# nb lignes
   nl <- dim(tab)[1]
# somme des lignes: margin.table(tab, 1) 
   if(nl==2) {
     P <- prod.S1(tab[1,1:nc-1],tab[2,2:nc])
   } else {
     P <- 0
     for(i in 1:(nl-2)) {
        P <- P + prod.S1(tab[i,1:nc-1],margin.table(tab[(i+1):nl,2:nc],2))
     }
     P <- P + prod.S1(tab[nl-1,1:nc-1],tab[nl,2:nc])
   }
   P
} 

Q.kendall <- function(tab) {
   
# nb colonnes
  nc <- dim(tab)[2]
# nb lignes
   nl <- dim(tab)[1]
# somme des lignes: margin.table(tab, 1) 
   if(nl==2) {
     Q <- prod.S1(tab[2,1:nc-1],tab[1,2:nc])
   } else {
     Q <- 0
     for(i in 1:(nl-2)) {
        Q <- Q + prod.S1(margin.table(tab[(i+1):nl,1:nc-1],2),tab[i,2:nc])
     }
     Q <- Q + prod.S1(tab[nl,1:nc-1],tab[nl-1,2:nc])
   }
   Q
} 

IPE.gamma <- function(tab) {
   P <- P.kendall(tab)
   Q <- Q.kendall(tab)
   (P-Q)/(P+Q)
}

IPE.delta <- function(tab,dim=1) {
   S <- S.kendall(tab)
   n <- margin.table(tab)
   
   if(dim == 1) {
# ligne t
      v <- margin.table(tab,1)
   } else if(dim == 2) {
      v <- margin.table(tab,2)
   } 
   if(dim<3) {
      return(2*S / (n*n - t(v) %*% v))
   } else {
      return( (IPE.delta(tab,1)+IPE.delta(tab,2))/2)
   }
}

IPE.tau.b <- function(tab) {
   S <- S.kendall(tab)
   n <- margin.table(tab)
   t <- margin.table(tab,1)
   u <- margin.table(tab,2)
   return(2*S / sqrt((n*n - t(t) %*% t)*(n*n - t(u) %*% u)))
}

IPE.tau.c <- function(tab) {
   S <- S.kendall(tab)
   m <- min(ncol(tab), nrow(tab))
   n <- margin.table(tab)
   return(2*S / (n*n - (m-1)/m)) 
}

IPE.Phi <- function(tab) { 
   par <- Chi2(tab)
   nc  <- par[1]
   nl  <- par[2]
   x   <- par[3]
   d <- nc * nl * (min(nc,nl)-1)
   return(sqrt(x/d))
}

IPE.C <- function(tab) { 
   par <- Chi2(tab)
   nc  <- par[1]
   nl  <- par[2]
   x   <- par[3]
   d <- nc * nl 
   return(sqrt(x/(x+d)))
}

effet<- function(tab,coef,dim=1){
    if(any (coef == c("gamma","delta","Phi","C","tau.b","tau.c"))) {
       cat(coef," = ",switch(coef,
          gamma = IPE.gamma(tab),
          delta = IPE.delta(tab,dim),
          Phi   = IPE.Phi(tab),
          C     = IPE.C(tab), 
          tau.b = IPE.tau.b(tab),
          tau.c = IPE.tau.c(tab)))
    } else {
       cat(coef,": coefficient inconnu")
    }
}

prod.S1<- function(v,w) {
   n <- length(v) 
   m2 <- matrix(rep(1,n*n),n,n,byrow=T)
   m2[lower.tri(m2)] <- 0
   t(v) %*% m2 %*% w
}

var.S <- function(tab) {
# nb colonnes
   nc <- dim(tab)[2]
# nb lignes
   nl <- dim(tab)[1]
# somme des lignes
   t <- margin.table(tab, 1)
# somme des colonnes
   u <- margin.table(tab, 2)
# total
   n <- sum(u)

   v <- (2*(n^3-sum(t^3) - sum(u^3)) + 3*(n^2 - sum(t^2)  - sum(u^2)) + 5*n) / 18
   v <- v + (sum(t^3) - 3*sum(t^2) + 2*n)*(sum(u^3) -3*sum(u^2) + 2*n)/ (9*n*(n-1)*(n-2))
   v <- v + (sum(t^2) - n)*(sum(u^2) - n)/(2*n*(n-1))
   v
}

correc.S <- function(tab) {
   corr <- 0
# nb colonnes
   nc <- dim(tab)[2]
# nb lignes
   nl <- dim(tab)[1]
# somme des lignes
   t <- margin.table(tab, 1)
# somme des colonnes
   u <- margin.table(tab, 2)
   if(isTRUE(u == 1)) {
      corr <- 1
   } else {
      if(nl==2) {
        corr <- (2*sum(t) - u[1]-u[nc]) / (2*(nc-1))
      } else {
         if(nl <= 3) {
            corr <- 1
         } else {
            corr <- 0.5
         }
      }
   }
   corr
}

S.test <- function(tab) {
# a brancher sur test exact pour des petits nombres
# a verifier que ce n'est pas un dataframe
   if(is.data.frame(tab)) tab <- as.matrix(tab)
   S <- S.kendall(tab)
   c <- correc.S(tab) 
   v <- var.S(tab)
   s <- sqrt(v)

   if(S>0) {
     Sc <- S-c   
   } else {
     Sc <- S+c
   }

   z <- Sc / s

   p <- pnorm(z)
   if(S>0) {
     p<- 1-p
   }

   cat("S =",S)
   cat(", corr =", c)
   cat(", Sc =",Sc)
   cat("\nvariance =",v)
   cat(", ecart-type =",format(s,digits=4))
   cat("\nz =",format(z,digits=3))
   cat(", p-value =",format(p,digits=2))
}

cree.table.old <- function(n,m,lrow=NULL,lcol=NULL) {
   mat <- matrix(rep(0,n*m),nrow=n,ncol=m,byrow=T)
   edit(mat)
}

cree.table <- function(n,m) {
  okrow <- F
  okcol <- F
  if(length(n)>1) {
    lrow <- n 
    n <- length(n) 
    okrow <- T 
  }
  if(length(m)>1) {
    lcol <- m 
    m <- length(m) 
    okcol <- T
  }
  mat <- matrix(rep(0,n*m),nrow=n,ncol=m,byrow=T)
  if(okcol) {colnames(mat) <- lcol}
  if(okrow) {rownames(mat) <- lrow}
  edit(mat)
}

cree.pattern <- function(t,u) {
# u somme des colonnes, t somme des lignes
# m nombre de colonnes, n nombre de lignes
   n <- length(t)
   m <- length(u)
   l <- list()

   if(n==2) {
      aplacermax <- min(u[1],t[1])
      aplacermin <- u[1]-min(u[1],t[2])

#     cat(m, "/", aplacermin, "/" , aplacermax, "/")
      if(m==1) {
         l[[1]] <- matrix(t,nrow=n,ncol=1,byrow=T) 
         return(l)
      } else {
         offset <- 0
         for(i in (aplacermin+1):(aplacermax+1)) {
            al <- cree.pattern(c(t[1]-i+1,t[2]-(u[1]-i+1)),u[2:m]) 
            for(j in 1:length(al)) {
               l[[offset+j]] <- cbind(c(i-1,u[1]-i+1), al[[j]])
            }
            offset <- offset+length(al)
         }
         return(l)
      }
   } else {
      offset <- 0
      al <- cree.pattern(c(t[1],sum(t[2:n])),u)
      for(i in 1:length(al)){
         tmp <- al[[i]]
         aal <- cree.pattern(t[2:n],tmp[2,])
         for(j in 1:length(aal)) {
            l[[offset+j]] <- rbind(tmp[1,],aal[[j]])
         }
         offset <- offset + length(aal)
      }
      return(l)
   }
}

cree.pattern2 <- function(t,m) {
# m nombre de colonnes t somme des lignes
   n <- length(t)
   l <- list()

   if(n==1 & m == 2) {
      for(i in 1:(t[1]+1)) {
         l[[i]] <- matrix(c(i-1,t[1]-i+1),nrow=1,ncol=2,byrow=T) 
       }
       return(l)
   } else if (n==1) {
      offset <- 0
      for(i in 1:(t[1]+1)) {
         al <- cree.pattern2(t[1]-i+1,m-1)
         for(j in 1:length(al)) {
            l[[j+offset]] <- cbind(c(i-1),al[[j]]) 
         }
         offset <- offset + length(al)
      } 
      return(l)
   } else {
      al1 <- cree.pattern2(t[1],m)
      al2 <- cree.pattern2(t[2:n],m)
      k <- 1         
      for(i in 1:length(al1)) {
         for(j in 1:length(al2)) {
            l[[k]] <- rbind(al1[[i]], al2[[j]])
            k <- k+1
         }
       }
       return(l)
   }
}

# test pour chi exact

moy.tab <- function(x){
   n  <- sum(x)
   cs <- colSums(x)
   rs <- rowSums(x)
   (rs %o% cs) / n
}

valeurs.chi <- function(){
  E <- (c(10,10) %o% c(5,6,9)) / 20
  l = list()
  i <- 1
  for(a in 1:10) {
    for(b in 1:(10-a)) {
       c = 10 - a - b
       d = 5-a
       e = 6-b
       f = 9-c 
   
       tmp <- matrix(c(a,b,c,d,e,f), nrow=2, ncol=3, byrow=T)
       l[[i]] = sum((tmp - E)^2 / E)
       i <- i+1
    }
  }
    l
}

# K

K.kruskal <- function(tab) {
   
# nb colonnes
   nc <- dim(tab)[2]
# nb lignes
   nl <- dim(tab)[1]

# somme des colonnes 
   u <- margin.table(tab,2)

# total
  n <- sum(u)

# rang moyen
  Rm <- rep(nc,0)
  
# facteur correctif
  Tc <- 0 
 
 etat <- 0
  for(i in 1:nc) {
     Rm[i] <- etat + (u[i]+1) / 2
     etat <- u[i]+etat
     Tc <- Tc + u[i] * (u[i]-1) * (u[i]+1)
  }
  Tc <- 1 - Tc/(n^3-n)

# somme des lignes: margin.table(tab, 1) 
   t <- margin.table(tab,1)
   R <- rep(nl,0)
   Rb <- rep(nl,0)

   Rcarre <- 0
   for(i in 1:nl) {
        R[i] <- tab[i,] %*% Rm
        if(t[i]==0) {
          Rb[i] = 0
        } else {
          Rb[i] <- R[i] / t[i]
        }  
      Rcarre <- Rcarre + R[i] * Rb[i]
   }
   K <- -3*(n+1) + (12 * Rcarre)/n/(n+1)

   c(nl,nc,K,Tc)
} 

K.test <- function(tab) {
# a brancher sur test exact pour des petits nombres
   par <- K.kruskal(tab)
   df <- (par[1]-1)
   Kc <- par[3]/par[4]
   cat("K =",par[3])
   cat(", T =", format(par[4],digits=3))
   cat(", Kc =",Kc)
   cat("\ndf =",df)
   cat(", p-value =",format(1-pchisq(Kc,df),digits=2))
}

# L Meddis

tab21.L <- matrix(c(1,0,1,0,0,0,1,0,1,0,0,
                    0,0,0,1,1,0,0,0,0,0,1,
                    0,1,0,0,0,1,0,1,0,1,0),ncol=11,nrow=3,byrow=T)

L.meddis <- function(tab,lambdas) {

# nb colonnes
  nc <- dim(tab)[2]
# nb lignes
  nl <- dim(tab)[1]

# somme des lignes et colonnes
  t <- margin.table(tab,1)
  u <- margin.table(tab,2)

# total
  n <- sum(t)

# rang moyen
  Rm <- rep(nc,0)
  
  etat <- 0
  T    <- 0
  for(i in 1:nc) {
    Rm[i] <- etat + (u[i]+1) / 2
    etat <- u[i]+etat
    T <- T + u[i]^3 - u[i]
  }

  R <- rep(nl,0)
  Rb <- rep(nl,0)

  for(i in 1:nl) {
    R[i] <- tab[i,] %*% Rm 
    Rb[i] <- R[i] / t[i]
  }

  T <- 1 - T/(n^3-n)
  L <- R %*% lambdas
  mu <- 0.5*(n+1)*t %*% lambdas
  va <- ((n+1)/12)*(n* t %*% lambdas^2 - (t %*% lambdas)^2)
  return(c(L,mu,va,T))  
}

tab25.L <- matrix(c(2,4,6,3,7,10,3,6,6,6,4,3), ncol=3,nrow=4,byrow=T)

L.test <- function(tab,lambdas) {
  par <- L.meddis(tab,lambdas)
  
  L  <- par[1]
  mu <- par[2]
  vL <- par[3]
  T  <- par[4]

  Delta  <- abs(L - mu) 
 
  sp <- sqrt(vL*T)
  z <- Delta/sp

  p <- 1-pnorm(z)
  cat("T = ",T," L = ",L,", mL = ",mu,",vL = ",vL,"\n")
  cat("sc = ",format(sp,trim=T,digits=5),", z = ",format(z,digits=3),", p-value = ",format(p,digits=2),"\n") 
}

# Chi2
Chi2 <- function(tab){
# nb colonnes
   nc <- dim(tab)[2]
# nb lignes
   nl <- dim(tab)[1]

# somme des colonnes et des lignes 
   u <- margin.table(tab,2)
   t <- margin.table(tab,1)
# total
   n <- sum(u)

# effectifs theorique
   Th <- t %*% t(u) /n
   Dif <- tab-Th
   Cdif<- Dif * Dif / Th

   Khi <- sum(Cdif)  
   c(nl,nc,Khi) 
}

Chi2.test <- function(tab) {
# prendre chisq.test pour les cas speciaux
   par <- Chi2(tab)
   df <- (par[2]-1)*(par[1]-1)
   cat("Chi2 =",par[3])
   cat("\ndf =",df)
   cat(", p-value =",1-pchisq(par[3],df))
}

multinomial <- function(n,decomp) {
   decomp <- sort(decomp,decreasing = TRUE)
   len <- length(decomp)
   if(sum(decomp)==n) {
     len <- len-1
     decomp <- decomp[1:len]
   }
   multi <- 1
   m <- n
   for(i in 1:len) {
     k <- decomp[i]
     multi <- multi * choose(m,k)
     m <- m - k
   }
   multi
}

multiplicite <- function(tab) {
   mult <-1
   nc <- dim(tab)[2]

   for(i in 1:nc) {
      decomp <- tab[,i]
      mult <- mult * multinomial(sum(decomp),decomp)
   }
   mult
}

distrib0.S <- function(ltab) {
   lS <- vector()
   lM <- vector()
   
   Stmp <- S.kendall(ltab[[1]])
   lS[1] <- S.kendall(ltab[[1]])
   lM[1] <- multiplicite(ltab[[1]])
   j <- 1
 
   for(i in 2:length(ltab)) {
      Scrt <-  S.kendall(ltab[[i]])
 #     if(Scrt == Stmp) {
 #       lM[j] <- lM[j] + multiplicite(ltab[[i]])
 #     } else {
        j <- j+1
        lS[j] <- S.kendall(ltab[[i]])
        lM[j] <- multiplicite(ltab[[i]])
#        Stmp <- Scrt
#      }
   }
   matrix(c(lS,lM),nrow=2, ncol=length(lS), byrow=T)    
}

distrib.exacte.S <- function(t,u=NULL) {
  if(is.null(u)) {
    return(regroupe.tab(distrib0.S(cree.pattern(margin.table(t,1),margin.table(t,2)))))
  } else {    
    return(regroupe.tab(distrib0.S(cree.pattern(t,u))))
  }
}

regroupe.tab <- function(tab) {
  lS <- vector()
  lM <- vector()
  ordre = order(tab[1,])
  j <- 1
  lS[1] <- tab[1,ordre[1]]
  lM[1] <- tab[2,ordre[1]]

  for(i in 2:length(ordre)) {
    ind <- ordre[i]
#    print(ind)
    if(lS[j] == tab[1,ind]) {
      lM[j] <- lM[j] + tab[2,ind]
    } else {
      j <- j+1
      lS[j] <- tab[1,ind]
      lM[j] <- tab[2,ind]
    }
  }
  return(matrix(c(lS,lM),nrow=2, ncol=length(lS), byrow=T))    
}

jeu1.test.K <- matrix(c(4,10,13,24,21,22,57,40,16),nrow=3, ncol=3, byrow=T)
val.jeu1 <- "K= 16.58; T= 0.87; df= 2; p < 0.001"

jeu2.test.S <- matrix(c(2,10,10,7,12,5),nrow=2, ncol=3, byrow=T)
val.jeu2 <- "S= -176; corr= 17; sigma=83.8, p = 0.029"


distrib0.Z <- function(tab,a) {
# la ligne i de tab contiennent la place du sujet i
# somme des lignes n1,n2,...,nr
# a ampant ; n nombre de sujets
n <- sum(tab) 
lst <- list()
if(n==1){
   r <- length(tab)
   v <- tab[1]
   j <- 1 
   while(v==0) {
      j <- j+1
      v <- tab[j] 
   }
   for(i in j:min(j+a,r)) {
#      lst[[i-j]] <- matrix(c(rep(0,i-1),1,rep(0,r-i)),nrow=1,ncol=r,byrow=T)
      lst[[i-j+1]] <- c(rep(0,i-1),1,rep(0,r-i))
   }
   return(lst)
} else {
   r <- dim(tab)[2]
   al1 <- distrib0.Z(tab[1,],a)
   al2 <- distrib0.Z(tab[2:n,],a)
      k <- 1         
      for(i in 1:length(al1)) {
         for(j in 1:length(al2)) {
            lst[[k]] <- rbind(al1[[i]], al2[[j]])
            k <- k+1
         }
       }
       return(lst)
}
}
 
somme.rang <- function(tab) {
   r <- dim(tab)[2]
   soml <- margin.table(tab,2)

   tot <- 0
   for(i in 1:r){
      tot <- tot + soml[i] * i
   }
}

tab26.K <- matrix(c(1,4,8,
                    3,6,5,
                    13,9,12,
                    11,12,10,
                    35,15,6,
                    22,25,10),ncol=3,nrow=6,byrow=T)

as.boite.old <- function(tab,lplaque,lrow,lcol){
  np <- length(lplaque)
  nl <- length(lrow)
  nc <- length(lcol)

  l <- list()

  l[[1]] <- c(np,nl,nc)
  l[[2]] <- lplaque
  l[[3]] <- lrow
  l[[4]] <- lcol

  off <- 1
  for(i in 1:np) {
    off1 <- off+nl-1
    l[[4+i]] <- matrix(tab[off:off1,], nrow=nl, ncol=nc, byrow=F)
    rownames(l[[4+i]]) <- lrow 
    colnames(l[[4+i]]) <- lcol 
    off <- off1+1
  }
  return(l)
}

aff.boite <- function(boite) {
  return(boite[5:length(boite)])
}

as.boite <- function(tab,lplaque,lrow,lcol){
  np <- length(lplaque)
  nl <- length(lrow)
  nc <- length(lcol)

  l <- list()

  l[[1]] <- c(np,nl,nc)
  l[[2]] <- lplaque
  l[[3]] <- lrow
  l[[4]] <- lcol

  off <- 1
  for(i in lplaque) {
    off1 <- off+nl-1
    l[[i]] <- matrix(tab[off:off1,], nrow=nl, ncol=nc, byrow=F)
    rownames(l[[i]]) <- lrow 
    colnames(l[[i]]) <- lcol 
    off <- off1+1
  }
  return(l)
}

tabSimp.S <- matrix(c(9,2,51,31,5,31,4,35),nrow=4,ncol=2,byrow=T)
tabSimp.S <- as.boite(tabSimp.S,c("Informatique","Economie"),
                                c("Filles","Garcons"),c("Reussite","Echec"))

tab24.ga <- matrix(c(2,4,6,3,7,10),nrow=2,ncol=3,byrow=T)
tab24.fi <- matrix(c(3,6,6,6,4,3),nrow=2,ncol=3,byrow=T)
tab24.F <- matrix(c(2,4,6,3,6,6),nrow=2,ncol=3,byrow=T)
tab24.D <- matrix(c(3,7,10,6,4,3),nrow=2,ncol=3,byrow=T)


S.ftest <- function(boite,plaque2=null){
  if(is.list(boite)) {
    plaque1 <- boite[[5]]
    plaque2 <- boite[[6]]
  } else {
    plaque1 <- boite
  }
  S1 <- S.kendall(plaque1)
  t1 <- margin.table(plaque1,1)
  n1 <- sum(t1)  
  f1 <- 2/(n1*n1 - t(t1) %*% t1)
  v1 <- var.S(plaque1)

  delta1 <- S1 * f1

  S2 <- S.kendall(plaque2)
  t2 <- margin.table(plaque2,1)
  n2 <- sum(t2)  
  f2 <- 2/(n2*n2 - t(t2) %*% t2)
  v2 <- var.S(plaque2)

  delta2 <- S2 * f2

  Ddelta <- abs(delta1-delta2)
  v <- v1*f1^2 + v2*f2^2
  z <- Ddelta/sqrt(v)
  p <- 1-pnorm(z)

  cat("\nAnalyse de l'interaction avec delta")  
  cat("\n***********************************\n")
  cat("D1 = ",format(delta1,trim=F,digits=3)," D2 = ",format(delta2,trim=F,digits=3) ,"\n")
  cat("delta = ",format(Ddelta,trim=F,digits=3)," v = ",format(v,trim=F,digits=3),"\n")
  cat("z = ",format(z,trim=F,digits=3)," p-value = ",format(p,trim=F,digits=3),"\n")

}

# Cas particulier pour chapitre III.12 : 
# donnees liste de tables avec 4 lignes (en fait 2 tables de 2 lignes) tabH et tabB
# on calcule multiplicite des deux tables et on multiplie: mH * mB = mt 
# on calcule delta et difference dH et dB

lstIII.12 <- list(matrix(c(1,2,0,1,0,1,1,0,1,0,2,0),nrow=4,ncol=3,byrow=T),
+ matrix(c(1,1,1,1,1,0,1,1,0,0,1,1),nrow=4,ncol=3,byrow=T),
+ matrix(c(2,0,1,0,2,0,0,2,0,1,0,1),nrow=4,ncol=3,byrow=T),
+ matrix(c(2,1,0,0,1,1,0,1,1,1,1,0),nrow=4,ncol=3,byrow=T))

lfIII.12 <- function(tab) {
    tabH <- tab[c(1,2),]
    tabB <- tab[c(3,4),]
    mH <- multiplicite(tabH)
    dH <- IPE.delta(tabH)
    mB <- multiplicite(tabB)
    dB <- IPE.delta(tabB)
    c(mH*mB,dB-dH)
}

fIII.12 <- function(lst) {
# lst creee par cree.pattern(c(3,2,2,2),c(3,4,2)) ou cree.pattern2(c(3,2,2,2),3) 
  lapply(lst,lfIII.12)
}

lanalIII.12 <- function(v,a) {
  if(v[2] >= a)  { 
    t <- v[1]
  } else {
    t <- 0
  }
  t
}

analIII.12 <- function(res,a) {
  do.call(sum,lapply(res,function(x) {lanalIII.12(x,a) }))
}

tab26.K <- as.boite(tab26.K,c("S","M","I"),c("F","D"),c("NC","I","C"))

K.ftest <- function(boite,np=NULL,nl=NULL,nc=ncol(boite)) {

  if(!is.list(boite)) {   
    if(is.null(np) | is.null(nl) | is.null(nc)) {
      cat("Parametres non definis\n")
      return(FALSE)
    }
    if(length(np) < 2 & length(nl) < 2 & length(nc) < 2) {
      if(nrow(boite) != np*nc | ncol != nc){       
        cat("Parametres errones\n")
        return(FALSE)
      }
    }

    if(length(np)<2) {
       np <- paste("P",seq(1:np),sep="")
    }
    if(length(nl)<2) {
       nl <- paste("L",seq(1:nl),sep="")
    }
    if(length(nc)<2) {
       nc <- paste("C",seq(1:nc),sep="")
    }
    boite <- as.boite(boite,np,nl,nc) 
  }

  np <- boite[[1]][1]
  nl <- boite[[1]][2]
  nc <- boite[[1]][3]

  plabel <- boite[[2]]
  rlabel <- boite[[3]]

  tabt <- boite[[5]]

  if(np == 1) {
    K.test(tabt)    
  } else {
    for(i in 6:(4+np)) {
      tabt <- rbind(tabt,boite[[i]])
    }

    tab1 <- boite[[5]][1,]
    for(i in 6:(4+np)) {
      tab1 <- tab1 + boite[[i]][1,]
    }

    for(j in 2:nl) {
      tab1 <- rbind(tab1,boite[[5]][j,])
      for(i in 6:(4+np)) {
        tab1[j,] <- tab1[j,] + boite[[i]][j,]
      }
    }

    tab2 <- margin.table(boite[[5]],2)
    for(i in 6:(4+np)) {
      tab2 <- rbind(tab2,margin.table(boite[[i]],2))
    }

    K  <- K.kruskal(tabt)
    df <- (K[1]-1)
    Kc <- K[3]/K[4]
    dfi <- df
    Ki <- Kc
    cat("\nAnalyse de variance non parametrique")
    cat("\n************************************\n")
    cat("T = ",format(K[4],trim=F,digits=2)," (facteur correctif)\n")
    cat("Effet total\n***********\n")
    cat("Ktotal = ",format(Kc,trim=T,digits=4)," (",format(K[3],trim=T,digits=4),"), df = ",df)
    cat(", p-value = ",format(1-pchisq(Kc,df),trim=T,digits=2),"\n")

    cat("Effet ",plabel,"\n*************\n")
    K <- K.kruskal(tab2)
    df <- (K[1]-1)
    Kc <- K[3]/K[4]
    dfi <- dfi - df
    Ki  <- Ki - Kc 
    cat("Kp = ",format(Kc,trim=T,digits=4)," (",format(K[3],trim=T,digits=4),"), df = ",df)
    cat(", p-value = ",format(1-pchisq(Kc,df),trim=T,digits=2),"\n")

    cat("Effet ",rlabel,"\n************\n")
    K <- K.kruskal(tab1)
    df <- (K[1]-1)
    Kc <- K[3]/K[4]
    dfi <- dfi - df
    Ki  <- Ki - Kc 
    cat("Kr = ",format(Kc,trim=T,digits=4)," (",format(K[3],trim=F,digits=4),"), df = ",df)
    cat(", p-value = ",format(1-pchisq(Kc,df),trim=T,digits=2),"\n")
    cat("Effet interaction","\n*******************\n")
    cat("Kint = ",format(Ki,trim=T,digits=4),", df = ",dfi)
    cat(", p-value = ",format(1-pchisq(Ki,df),trim=T,digits=2),"\n")

  }
}

distrib1.Z <- function(tab,a){
   lst <- distrib0.Z(tab,a)
   ref <- somme.rang(tab)
   vec <- rep(0,length(lst))
   for(i in 1:length(lst)) {
      vec[i] <- somme.rang(lst[[i]])-ref
   } 
   return(table(vec))
}

distrib.exacte.Z <- function(v,a) {
   return(distrib1.Z(enumLM(v),a))
}

enumLM <- function(v) {
# passage d une enumeration sous forme matricielle
# c(2,1,1)  -> matrix(c(1,0,0,1,0,0,0,1,0,0,0,1), nrow=3)
n <- length(v)
s <- sum(v)
tab <- matrix(rep(0,s),nrow=s, ncol=n, byrow=T)

i <- 1
for (j in 1:n) {
   for(k in 1:v[j]) {
      tab[i,j] <- 1
      i <- i+1
   }
}
return(tab)
}

distrib.Z <- function(tab,v,a, pct=TRUE){
# tab tableau des distribution Zi sous la forme de poids si pct=FALSE
# v effectifs "avant" = c(n1,n2,... nr); a ampant  
# retourne moyenne et ecart-type

  r <- length(v)
  tabZ <- matrix(rep(0,2*(r-1)),nrow=r-1,ncol=2,byrow=T)

  if(pct & (sum(tab[1,])>1)) {
     pct <- F
     cat("Attention au parametre  pct \n")
  }
# traitement des Zi
  for(i in 1:(r-1)) {
     if(pct) {
        m <- 0
        var <- 0
        for(j in 1:min(a,r-i)+1){      
           m <- m + (j-1)*tab[i,j]
           var <- var + (j-1)*(j-1)*tab[i,j]
        }
        var <- var - m*m
        tabZ[i,1] <- m # moyenne
        tabZ[i,2] <- var # variance
     } else {
        for(i in 1:(r-1)){
           tab[i,] <- tab[i,]/sum(tab[i,])
        }
        return(distrib.Z(tab,v,a,T))
     }  
  }
  m <- 0
  var <- 0
  for(i in 1:(r-1)){
     m <- m + tabZ[i,1] * v[i]
     var <- var + tabZ[i,2] * v[i]
  }
  return(c(m,sqrt(var)))
}

Z.test <- function(tab,av,ap,a,pct=TRUE){
   dif <- (ap-av) %*% seq(0:(length(av)-1))
   coord <- distrib.Z(tab,av,a,pct)
   z <- (dif-coord[1])/coord[2]
   cat("moy: ",coord[1],"; ecart-type: ",coord[2],"\n") 
   cat("diff: ", dif, "; z: ", z, "; p-value: ",1-pnorm(z)) 
}
 
# distrib.Z(matrix(c(1/3,1/3,1/3,1/2,1/2,0),nrow=2,ncol=3,byrow=T),c(2,1,1),2)

cronbach <- function (tab) {
   tab <- na.omit(tab)
   nc <- ncol(tab)
   nl <- nrow(tab)
   alpha <- (nc/(nc - 1)) * (1 - sum(apply(tab,2,var))/var(apply(tab,1,sum)))
   return(list(sample.size = nl, number.of.items = nc, alpha = alpha))
}

tab.cronbach <- matrix(c(1,1,1,1,
                         1,1,1,2,
                         1,1,2,3,
                         1,2,2,4,
                         1,2,2,5,
                         2,2,3,1),ncol=4,nrow=6,byrow=T)

pct.table <- function(tab,pct="row") {
# Totaux marginaux
   tot.c <- margin.table(tab,2)
   tot.r <- margin.table(tab,1)
   n <- sum(tot.c)

# Calcul des pourcentages
   if(pct=="col") tpct <- tab %*% diag(1/tot.c);
   if(pct=="row") tpct <- diag(1/tot.r) %*% tab ;
   if(pct=="total") tpct <- tab / n
   tpct
}

pretty.table.old <- function(tab,pct="row") {
# Version 1.0, (c) IRDP, 2009
# Dimension
   nr <- nrow(tab) 
   nc <- ncol(tab)
   n <- nr*nc

# Totaux marginaux
   tot.c <- margin.table(tab,2)
   tot.r <- margin.table(tab,1)
   tot   <- sum(tab)

# Calcul des pourcentages
   if(pct=="col") tpct <- tab %*% diag(1/tot.c);
   if(pct=="row") tpct <- diag(1/tot.r) %*% tab ;

# Constitution du tableau
   for(i in 1:nr) for(j in 1:nc) {tab[i,j] <- paste(format(tab[i,j],digit=3)," (",format(tpct[i,j],digit=3),")",sep="")}
   tab <- cbind(tab, tot.r)
   rbind(tab, tot.c=c(tot.c,tot))
}

pretty.table <- function(tab,pct="row") {
# Version 1.1, (c) IRDP, 2009
# Dimension
   nr <- nrow(tab) 
   nc <- ncol(tab)
   n <- nr*nc
   dimn <- dimnames(tab)

# Totaux marginaux
   tot.c <- margin.table(tab,2)
   tot.r <- margin.table(tab,1)
   tot   <- sum(tab)

# Calcul des pourcentages
   if(pct=="col") tpct <- tab %*% diag(1/tot.c);
   if(pct=="row") tpct <- diag(1/tot.r) %*% tab ;
   if(pct=="total") tpct <- tab/tot ;

# Constitution du tableau
#   tab <- paste(format(tab,digit=3)," (",format(tpct,digits=3),")",sep="")
   tab <- paste(tab," (",format(tpct,digits=3),")",sep="")
   tab <- matrix(tab,nrow=nr,ncol=nc,byrow=F,dimnames=dimn)
   tab <- cbind(tab, tot.r)
   rbind(tab, tot.c=c(tot.c,tot))
}

# Outils pour les graphes repris
# VersionHtxtRD <- "v1 avril 2010" (pochon & favre)
 
fermeture <- function(gb,thres=0,arr=T) {
  n  <- dim(gb)[1]
  u  <- diag(n)
  gb <- gb %*% solve(u-gb/n)

  if(thres != 0) {
     gb[gb < thres] <- 0
  }
  if(arr) {
    gb[gb > 0] <- 1
  }
  return(gb)
}

mcore <- function(gb,thres=0,ferm=F) {
# CORE a partir matrice d'adjacence qcq
# ferm indique si la fermeture transitive est faite ou non
  if(!ferm) {
    gb <- fermeture(gb,thres,arr=T)
  }
  (gb+t(gb)) %/% 2
}

core <- function(gb,thres=0,ferm=F) {
  margin.table(mcore(gb,thres,ferm),1)
}

centralite <- function(gb,thres=0,ferm=F,arr=F) {
# centralite pour une matrice d'adjacence qcq
# ferm indique si la fermeture a deja ete faite
  n  <- dim(gb)[1]

  if(!ferm) {
    gb <- fermeture(gb,thres,arr)
  }

  aux <- matrix(rep(0,2*n),ncol=n,nrow=2)
# sortante
  aux[1,] <- margin.table(gb,1) 
# entrante
  aux[2,] <- margin.table(gb,2)
  return(aux)
}

# Calcul entropie

HP <- function(v,relative=F) {
    N <- sum(v)
    v <- v / N
    ent = -sum(v*log(v))
    if(relative) {
        ent = ent / log(N)
    }
    return(ent)
} 

H <- function(v,relative=F) {
    if(relative) {
        ent = HP(v,T)
    } else {
        ent = HP(v) / log(2)
    }
    return (ent)
}

HH <- function(v, exact= T) {
    N <- sum(v)
    v <- v/N
    if(exact) {
        ent = N * (sum(v*v) - 1/N) / (N-1) 
    } else {
        ent = sum(v*v)
    }
    return(ent)
}   

# entropie de Boltzmann

SB <- function(v) {
     N <- sum(v)
     v <- gamma(v+1) ;
     return (log(gamma(N+1) / produit(v)) /N)
}
    

produit <- function(v) {
    p <- 1;
    for(i in 1:length(v)){
        p <- p * v[i] 
    }
    return (p);
}

HV <- function(v,x) {
    v <- v*x
    return(SB(v))
}

